#! /usr/bin/python
# -*- coding: utf-8 -*-

# this is for solr queries
#
# author Illina Yang
# Wed Aug 4 11:: PDT 2014

# Rewrite for xena 0.2.0
# Thu Aug 14 11:13:38 PDT 2014
#
# Notes:
# This version is substantially faster, because the patient query has
# been rewritten to be much more efficient.
#
# Additionally, this version only sends unique patient IDs to the
# server, since it wastes time to send duplicates.
# 
# json and post calls have been removed from the xena methods, so they
# are transport and encoding agnostic. Convenience methods that perform
# json and post calls are derived near the top of this script.

VERSION="3.0"

import re
import urllib2
from urllib2 import *
import sys, getopt
import xena_query as xena
import json
from functools import partial
from itertools import groupby

def merge_dicts(l):
    """Merge a list of dictionaries"""
    return {k: v for d in l for k, v in d.items()}

#xena_url= "https://genome-cancer.ucsc.edu/proj/public/xena"
xena_url= "http://tcga1:7223"

# convenience aliases for xena methods.
# xena.compose is a functional composition, provided here to avoid
# dependency on 
post = xena.compose(json.loads, partial(xena.post, xena_url))
find_cohorts = partial(post, xena.cohort_query_str)
patient_to_sample_query = xena.compose(post, xena.patient_to_sample_query)

def usage(exit_code=1):
  print >> sys.stderr, "Usage: " + sys.argv[0]
  print "	Enter keywords for search (case insensitive)."
  print "VERSION: " + VERSION
  sys.exit(exit_code);

INPUT = sys.argv[1:]
if len(INPUT) == 0:
  usage()
WORDS = ' '.join(sys.argv[1:])	#command line text after script name
LINK = ''

WORDS = WORDS.strip()
WORDS = WORDS.replace(' ','+AND+')

LINK = "http://kolossus.cse.ucsc.edu:8983/solr/collection1/select?q=" + WORDS + "&rows=14986&fl=cohort%2C+patient&wt=python&indent=true"

connection = urlopen(LINK)
response = eval(connection.read())
print response['response']['numFound'], "docs found"

by_cohort =  lambda x: x['cohort']

# cohorts in xena
cohorts = find_cohorts()

# util to normalize cohort name
cohorts_normalized = {k.lower(): k for k in cohorts}
normalize_cohort = lambda c: cohorts_normalized.get(c.lower(), c)

docs = response['response']['docs']
# solr results with normalized cohort name
docs_normalized = [merge_dicts((doc, {'cohort': normalize_cohort(doc['cohort'])}))
        for doc in docs]

# group docs by cohort
docs_by_cohort = groupby(sorted(docs_normalized, key=by_cohort), by_cohort)
# extract patient from docs, and make unique via set
patients_by_cohort = {cohort: set([doc['patient'] for doc in docs])
        for (cohort, docs) in docs_by_cohort}

# intersect with xena cohort list to make list of requests
requested = {cohort: patients for (cohort, patients) in patients_by_cohort.items()
        if cohort in cohorts}

results = merge_dicts([patient_to_sample_query(cohort, patients) 
    for (cohort, patients) in requested.items()])

for (cohort, patients) in results.items():
    print cohort, len(patients)

def log_missing(file, cohort, patients):
    msg = cohort + " is not in the Xena server. " + str(len(patients)) + " patient IDs were not included."
    file.write(msg + "\n")
    print msg

missing_cohort = [cohort for cohort in patients_by_cohort.keys()
        if cohort not in cohorts]

f = open("error.log", "wb")

for cohort in missing_cohort:
    log_missing(f, cohort, patients_by_cohort[cohort])
